#!/bin/python3

import sys
import random


import argparse

parser = argparse.ArgumentParser(description = "A program running genetic algorithms")
parser.add_argument('popsize', help= "Population size", type=int)
parser.add_argument('surv_frac', help= "Surviving fraction", type=float)
parser.add_argument('mut_prob', help= "Mutation probability", type=float)
parser.add_argument('ngen', help= "N of generations", type=int)
parser.add_argument('--no_plots', help="does not show plots", default=False, action='store_true')
args = parser.parse_args()

print(args)

npop = args.popsize
surv_prob = args.surv_frac
mut_prob = args.mut_prob
ngen = args.ngen


#npop = int(sys.argv[1])

#surv_prob = float(sys.argv[2])

#mut_prob = float(sys.argv[3])

#ngen = int(sys.argv[4])

def binarytoint(ll):
    """ a function that converts a list of 0/1 
    into a decimal integer
    """
    res = 0
    base = 1
    for i in reversed(ll):
        res+= i*base
        base*=2

    return res



class Individual(list):

    def get_fitness(self):
        return self.get_green() - self.get_red() - self.get_blue() + 512

    def get_red(self):
        return binarytoint(self[0:8])

    def get_green(self):
        return binarytoint(self[8:16])

    def get_blue(self):
        return binarytoint(self[16:])

    def mate(self, other, crossover = None):
        if not crossover:
            crossover = random.randint(0, len(self) - 1)
        return Individual(self[:crossover] + other[crossover:])

    def mutate(self, prob):
        if random.random() < prob:
            pos = random.randint(0, len(self)-1)
            self[pos] = random.randint(0,1)


def generate_population(npop):
    population = []
    for i in range(npop):
        population.append(Individual([random.randint(0,1) for j in range(24)]))
    return population



def get_best_individuals(pop, frac):
    pop.sort(key = lambda i : i.get_fitness(), reverse = True)
    return pop[:int(len(pop)*frac)]


def get_offsprings(parents, n, mut_prob):
    offsprings = []
    for i in range(n):
        parents1, parents2 = random.sample(parents, 2)
        offsprings.append(parents1.mate(parents2))
        offsprings[-1].mutate(mut_prob)
    return offsprings


import matplotlib.pyplot as plt
import matplotlib.patches as patches
fig, axes = plt.subplots(nrows = ngen)

def plot_color(pop, igen):
    ax = axes[igen]
    width = 20
    height = 1. /ngen
    origin = [0.,0.]

    for ind in pop:
        ax.add_patch(patches.Rectangle(tuple(origin), width, height, \
                facecolor = (ind.get_red()/255, ind.get_green()/255, ind.get_blue()/255.)))
        origin[0] += width

    ax.set_xlim([0, width*len(pop)])
    ax.set_ylim([0, height])





pop = generate_population(npop)
best_score = max([ind.get_fitness() for ind in pop])
print(best_score)

for i in range(ngen):
    plot_color(pop,i)
    parents = get_best_individuals(pop, surv_prob)
    offsprings = get_offsprings(parents, npop - len(parents), mut_prob)
    pop = parents + offsprings

    best_score = max([ind.get_fitness() for ind in pop])
    print(best_score)
    plt.pause(0.1)
    plt.draw()

plt.show()
